#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from homevee.Functions.sensor_data import get_sensor_value
from homevee.Helper import Logger, smarthome_functions


def conditions_true(condition_data, db):
    #Format: {'and':[{'and':[{'action':'time', 'comparator':'greater', 'value':'08:00'}, {'action':'time', 'comparator':'less', 'value':'20:00'}]},
    # {'and':[{}, {}]}]}

    if('and' in condition_data):
        for condition in condition_data['and']:
            if not conditions_true(condition, db):
                return False
            return True
    if(isinstance(condition_data, list)):
        if(len(condition_data) == 0):
            return True
        for condition in condition_data:
            if not conditions_true(condition, db):
                return False
            return True
    elif('or' in condition_data):
        for condition in condition_data['or']:
            if conditions_true(condition, db):
                return True
        return False
    else:
        #check single conditions
        return check_single_condition(condition_data, db)

def check_single_condition(condition_data, db):
    #{'action': 'timeperiod', 'from': '08:00', 'to': '20:00'}
    if(condition_data['type'] == "timeperiod"):
        return is_in_time_period(condition_data)
    #{'action': 'weekdays', 'days': [5, 6]}
    elif(condition_data['type'] == "weekdays"):
        return datetime.datetime.today().weekday() in condition_data['days']
    #{'action': 'sensor_value', 'type': 'MQTT Sensor', 'id': '1', 'comparator': '>', 'value': 28}
    elif(condition_data['type'] == "sensorvalue"):
        value = get_sensor_value(condition_data['devicetype'], condition_data['deviceid'], False, db)
        Logger.log(value)
        return check_value(condition_data['comparator'], value, condition_data['value'])
    #{'action': 'user_at_home', 'user': 'sascha', 'at_home': False}
    elif(condition_data['type'] == "user_at_home"):
        if(condition_data['at_home']):
            return smarthome_functions.user_is_at_home(condition_data['username'], db)
        elif(not condition_data['at_home']):
            return not smarthome_functions.user_is_at_home(condition_data['username'], db)
    return False

def check_value(comparator, is_value, value_condition):
    if(comparator == ">"):
        return float(is_value) > float(value_condition)
    elif(comparator == ">="):
        return float(is_value) >= float(value_condition)
    elif(comparator == "<"):
        return float(is_value) < float(value_condition)
    elif(comparator == "<="):
        return float(is_value) <= float(value_condition)
    elif(comparator == "=="):
        return float(is_value) == float(value_condition)
    elif(comparator == "!="):
        return float(is_value) != float(value_condition)

def is_in_time_period(condition_data):
    today = datetime.datetime.today()

    from_hour, from_minute = condition_data['from'].split(":")

    to_hour, to_minute = condition_data['to'].split(":")

    from_hour = int(from_hour)
    from_minute = int(from_minute)
    to_hour = int(to_hour)
    to_minute = int(to_minute)

    if (today.hour == from_hour):
        if (today.minute >= from_minute):
            if (today.hour == to_hour):
                if (today.minute <= to_minute):
                    return True
            elif (today.hour < to_hour):
                return True
    elif (today.hour >= from_hour):
        if (today.hour == to_hour):
            if (today.minute <= to_minute):
                return True
        elif (today.hour < to_hour):
            return True
    else:
        return False


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from homevee.cronjobs import FixedTimeCronjob
from homevee.utils.database import get_database_con
from homevee.utils.device_types import *


class SaveSensorDataCronjob(FixedTimeCronjob):
    def __init__(self):
        super(SaveSensorDataCronjob, self).__init__(task_name="SaveSensorDataCronjob")

    def task_to_do(self, *args):
        self.save_sensor_data()

    def get_seconds_to_wait(self, execution_time=None):
        t = datetime.datetime.today()

        seconds_to_wait = (60*60) - (t.minute*60) - t.second

        return seconds_to_wait

    def save_sensor_data(self):
        db = get_database_con()

        with db:
            cur = db.cursor()

            #Z-Wave Sensoren
            cur.execute("SELECT * FROM ZWAVE_SENSOREN")
            for item in cur.fetchall():
                if item['SAVE_DATA']:
                    self.save_to_db(ZWAVE_SENSOR, item['ID'], item['VALUE'], db)

            #MQTT Sensoren
            cur.execute("SELECT * FROM MQTT_SENSORS")
            for item in cur.fetchall():
                if item['SAVE_DATA']:
                    self.save_to_db(MQTT_SENSOR, item['ID'], item['LAST_VALUE'], db)
        db.close()

    def save_to_db(self,type, id, value, db):
        with db:
            time = datetime.datetime.now().strftime('%Y-%m-%d %H:00')

            cur = db.cursor()

            cur.execute("INSERT INTO SENSOR_DATA (DEVICE_ID, DEVICE_TYPE, DATETIME, VALUE) \
                            VALUES (:id, :type, :time, :value)",
                        {'id': id, 'type': type, 'time': time, 'value':value})


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.Helper.helper_functions import has_permission


def add_trigger(username, location, id, db):
    return

def get_triggers(username, room, db):
    if not has_permission(username, room, db):
        return {'result': 'nopermission'}

    triggers = []

    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM MQTT_TRIGGERS WHERE LOCATION = :location", {'location': room})

        for item in cur.fetchall():
            triggers.append({'name': item['NAME'], 'id': item['ID'], 'type': 'MQTT-Trigger', 'icon': item['ICON']})

    return {'triggers': triggers}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from homevee.DeviceAPI.heating import heating_control
from homevee.Helper import Logger
from homevee.cronjobs import IntervalCronjob
from homevee.utils.database import get_database_con


class HeatingSchemeCronjob(IntervalCronjob):
    def __init__(self):
        super(HeatingSchemeCronjob, self).__init__(task_name="HeatingScheme", interval_seconds=60)

    def task_to_do(self, *args):
        self.run_heating_scheme()

    def run_heating_scheme(self):
        db = get_database_con()

        current_datetime = datetime.datetime.now()

        with db:
            cur = db.cursor()

            #Query heating scheme for this day and time
            query = 'SELECT * FROM HEATING_SCHEME, HEATING_SCHEME_DAYS, HEATING_SCHEME_DEVICES WHERE HEATING_SCHEME.ID = HEATING_SCHEME_ID AND HEATING_SCHEME.ID = HEATING_SCHEME_DEVICES.ID AND ACTIVE = "true" AND TIME = :time AND WEEKDAY_ID = :day;'

            current_time = current_datetime.strftime("%H:%M")
            current_day = current_datetime.weekday()

            #Run command
            cur.execute(query, {'time': current_time, 'day': current_day})

            for item in cur.fetchall():
                Logger.log(item)

                heating_control(None, item['TYPE'], item['DEVICE_ID'], item['VALUE'], db, check_user=False)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import random
import traceback
import urllib.error
import urllib.parse
import urllib.request

from homevee.Helper import Logger
from homevee.VoiceAssistant import helper
from homevee.VoiceAssistant.Modules import VoiceModule
from homevee.VoiceAssistant.voice_patterns import PATTERN_JOKE


class VoiceGetJokesModule(VoiceModule):
    def get_pattern(self, db):
        return PATTERN_JOKE

    def get_label(self):
        return "joke"

    def run_command(self, username, text, context, db):
        return self.get_joke(username, text, context, db)

    def get_joke(self, username, text, context, db):
        try:
            url = helper.SMART_API_PATH + "/?action=joke&text=" + urllib.parse.quote(text.encode('utf8'))
            Logger.log(url)
            data = urllib.request.urlopen(url).read()

            data = data.decode('utf-8')

        except Exception as e:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            data = None

        if data is not None:
            return {'msg_speech': data, 'msg_text': data}
        else:
            result = self.get_error()
            return {'msg_speech': result, 'msg_text': result}

    def get_error(self):
        return random.choice([
            'Mir fällt gerade kein Witz ein.'
        ])